import sqlite3
import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.ticker as ticker
import seaborn as sns
from datetime import datetime, timedelta
import mpld3
from mpld3 import plugins

# Constants
USER_ID = 1
CATEGORY_TYPE = 'activity'
TIMEZONE = 'America/Toronto'
CUT_OFF_YEAR = 2012

MAIN_CATEGORIES = [
    {'name': 'A+', 'color': '#fdcce5'},
    {'name': 'Personal', 'color': '#beb9db'},
    {'name': 'Discretionary - Productive - Emacs', 'color': '#648caa'},
    {'name': 'Discretionary - Productive', 'color': '#7eb0d5'},
    {'name': 'Discretionary - Play', 'color': '#b2e061'},
    {'name': 'Discretionary - Social', 'color': '#ffee65'},
    {'name': 'Discretionary - Family', 'color': '#eedd65'},
    {'name': 'Unpaid work', 'color': '#ffb55a'},
    {'name': 'Business', 'color': '#fd7f6f'},
    {'name': 'Work', 'color': '#fd7f6f'},
    {'name': 'Sleep', 'color': '#cccccc'}
]

CATEGORY_NAMES = [cat['name'] for cat in MAIN_CATEGORIES]

def load_data(db_path):
    conn = sqlite3.connect(db_path)
    cursor = conn.cursor()

    query = """
    SELECT r.timestamp, r.end_timestamp, rc.full_name, rc.color
    FROM records r
    INNER JOIN record_categories rc ON r.record_category_id = rc.id
    WHERE r.user_id = ? AND rc.category_type = ? AND r.end_timestamp IS NOT NULL
    ORDER BY r.created_at DESC
    """
    cursor.execute(query, (USER_ID, CATEGORY_TYPE))
    data = cursor.fetchall()
    conn.close()

    df = pd.DataFrame(data, columns=['timestamp', 'end_timestamp', 'full_name', 'color'])
    df['timestamp'] = pd.to_datetime(df['timestamp'], format='mixed').dt.tz_localize('UTC').dt.tz_convert(TIMEZONE)
    df['end_timestamp'] = pd.to_datetime(df['end_timestamp'], format='mixed').dt.tz_localize('UTC').dt.tz_convert(TIMEZONE)
    df = df[df['timestamp'].dt.year >= CUT_OFF_YEAR]
    return df

def assign_main_category(full_name):
    for category in CATEGORY_NAMES:
        if full_name.startswith(category):
            return category
    return 'Other'

def calculate_time_by_category_and_year(df):
    time_segments = []
    for _, row in df.iterrows():
        start_time = row['timestamp']
        end_time = row['end_timestamp']
        full_name = row['full_name']
        main_category = assign_main_category(full_name)
        if end_time <= start_time:
            continue
        current_time = start_time
        while current_time.year < end_time.year:
            next_year = pd.Timestamp(year=current_time.year + 1, month=1, day=1,
                                    tz=current_time.tz)
            segment_seconds = (next_year - current_time).total_seconds()
            time_segments.append({
                'year': current_time.year,
                'category': main_category,
                'seconds': segment_seconds
            })
            current_time = next_year
        segment_seconds = (end_time - current_time).total_seconds()
        time_segments.append({
            'year': current_time.year,
            'category': main_category,
            'seconds': segment_seconds
        })
    segments_df = pd.DataFrame(time_segments)
    summary = segments_df.groupby(['year', 'category'])['seconds'].sum().reset_index()
    return summary

def calculate_hours_per_day(summary_df):
    yearly_totals = summary_df.groupby('year')['seconds'].sum().reset_index()
    yearly_totals.rename(columns={'seconds': 'total_seconds'}, inplace=True)
    summary_with_totals = pd.merge(summary_df, yearly_totals, on='year')
    summary_with_totals['percentage'] = summary_with_totals['seconds'] / summary_with_totals['total_seconds']
    summary_with_totals['hours_per_day'] = (summary_with_totals['percentage'] * 24).round(1)
    summary_with_totals.to_csv('hours-per-day.csv', index=False)
    return summary_with_totals# Save SVG
    output_svg = "hours-per-day.svg"


class SetViewbox(plugins.PluginBase):
    """Add a viewbox for better responsiveness"""
    JAVASCRIPT = """
    mpld3.register_plugin("setviewbox", SetViewbox);
    SetViewbox.prototype = Object.create(mpld3.Plugin.prototype);
    SetViewbox.prototype.constructor = SetViewbox;
    function SetViewbox(fig, props){
        mpld3.Plugin.call(this, fig, props);
    };

    SetViewbox.prototype.draw = function(){
        this.fig.canvas.attr("viewBox", "0 0 " + this.fig.width + " " + this.fig.height)
    }
    """
    def __init__(self):
        self.dict_ = {"type": "setviewbox"}

def create_visualization(summary_df):
    pivot_df = summary_df.pivot(index='year', columns='category', values='hours_per_day')
    plt.figure(figsize=(14, 8))
    sns.set_style('whitegrid')
    data_points = []
    for category_dict in MAIN_CATEGORIES:
        category = category_dict['name']
        color = category_dict['color']
        if category in pivot_df.columns:
            x_data = pivot_df.index
            y_data = pivot_df[category].values
            line = plt.plot(
                x_data,
                y_data,
                label=category,
                marker='o',
                markersize=6,
                linewidth=2,
                color=color
            )[0]
            for x, y in zip(x_data, y_data):
                if not pd.isna(y):
                    data_points.append({
                        'category': category,
                        'year': x,
                        'value': y,
                        'color': color
                    })
    plt.title('Average hours per day by category over time')
    plt.xlabel('Year')
    plt.ylabel('Average hours per day')
    plt.legend(loc='best', frameon=True)
    plt.grid(True)
    plt.gca().xaxis.set_major_formatter(ticker.StrMethodFormatter('{x:.0f}'))
    plt.tight_layout()
    plt.savefig("hours-per-day.svg", bbox_inches='tight')
    for i, category_dict in enumerate(MAIN_CATEGORIES):
        category = category_dict['name']
        color = category_dict['color']
        if category in pivot_df.columns:
            for line in plt.gca().get_lines():
                if line.get_label() == category:
                    tooltip_labels = []
                    x_vals = pivot_df.index
                    y_vals = pivot_df[category].values
                    for year, hours in zip(x_vals, y_vals):
                        if not pd.isna(hours):
                            tooltip_labels.append(f"{category} ({year}): avg {hours:.1f} hours/day")
                    tooltip = plugins.PointHTMLTooltip(line, tooltip_labels)
                    plugins.connect(plt.gcf(), tooltip)
    plugins.connect(plt.gcf(), SetViewbox())
    print(mpld3.fig_to_html(plt.gcf()))
    return plt

def main(db_path):
    df = load_data(db_path)
    summary = calculate_time_by_category_and_year(df)
    summary_with_hours = calculate_hours_per_day(summary)
    create_visualization(summary_with_hours)
    return summary_with_hours

if __name__ == "__main__":
    summary = main("production.sqlite3")
